import { HttpException, HttpStatus, Injectable } from "@nestjs/common";
import { InjectRepository } from "@nestjs/typeorm";
import { Constants } from "src/utils/constants";
import { ResponseUtils } from "src/utils/response.utils";
import { DoSpacesService } from "src/modules/SpacesModule/SpacesService/doSpacesService";
import { LocationPageCmsRepository } from "./location-page-cms.repository";
import { UpdateLocationPageCmsDto } from "./dto/update-location-page-cms.dto";
import { LocationPageCmsEntity } from "./entities/location-page-cms.entity";

@Injectable()
export class LocationPageCmsService {
  constructor(
    @InjectRepository(LocationPageCmsRepository)
    private readonly repository: LocationPageCmsRepository,
    private readonly doSpaceService: DoSpacesService
  ) {
  }

  async updateOrCreateData(
    dto: UpdateLocationPageCmsDto
  ): Promise<LocationPageCmsEntity | Error> {
    const isExists = await this.repository.find({ isDeleted: false });
    if (Object.keys(isExists).length === 0) {
      const data = await this.repository.createEntity(dto);
      if (!data) {
        throw new HttpException(
          Constants.NOT_FOUND,
          HttpStatus.NOT_FOUND
        );
      }
      return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
    } else {
      let id = isExists[0].id;
      const data = await this.repository.updateEntity(dto, id);
      if (!data) {
        throw new HttpException(
          Constants.UPDATE_FAILED,
          HttpStatus.BAD_REQUEST
        );
      }
      return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
    }
  }

  async findAll(): Promise<LocationPageCmsEntity[] | Error> {
    const data = await this.repository.findAll();
    if (!data) {
      throw new HttpException(
        Constants.NOT_FOUND,
        HttpStatus.NOT_FOUND
      );
    }
    return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
  }

  async remove(): Promise<boolean | Error> {
    const isExists = await this.repository.find({ isDeleted: false });
    if (Object.keys(isExists).length != 0) {
      let id = isExists[0].id;
      const data = await this.repository.deleteEntity(id);
      if (!data) {
        throw new HttpException(
          Constants.NOT_FOUND,
          HttpStatus.NOT_FOUND
        );
      }
      return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
    }
  }
}