import { EntityRepository, getConnection, Repository } from "typeorm";
import { CreateBlogCategoryDto } from "./dto/create-blog-category.dto";
import { UpdateBlogCategoryDto } from "./dto/update-blog-category.dto";
import { BlogCategory } from "./entities/blog-category.entity";

@EntityRepository(BlogCategory)
export class BlogCategoryRepository extends Repository<BlogCategory> {

  async createEntity(dto: CreateBlogCategoryDto): Promise<BlogCategory | Error> {
    try {
      const { categoryName } = dto;
      const entity = this.create({
        categoryName
      });
      await this.save(entity);
      return entity;
    } catch (e) {
      return e;
    }
  }

  async findByFilterQuery(query: any): Promise<BlogCategory[] | Error> {
    try {
      return await this.find({
        where: { ...query, isDeleted: false }
      });
    } catch (e) {
      return e;
    }
  }

  async findOneEntity(id: string): Promise<BlogCategory | Error> {
    try {
      return await this.findOne({ where: { id: id } });
    } catch (e) {
      return e;
    }
  }

  async updateEntity(id: string, dto: UpdateBlogCategoryDto): Promise<BlogCategory | Error> {
    try {
      await this.update(id, dto);
      return await this.findOne({ where: { id: id } });
    } catch (e) {
      return e;
    }
  }

  async deleteEntity(id: string): Promise<boolean | Error> {
    try {
      const updateResult = await getConnection()
        .createQueryBuilder()
        .update(BlogCategory)
        .set({ isDeleted: true })
        .where("id = :id", { id: id })
        .execute();
      if (updateResult.affected > 0) {
        return true;
      }
      return false;
    } catch (e) {
      return e;
    }
  }

  async findCategoryByCategoryName(categoryName: string): Promise<BlogCategory[] | Error> {
    try {
      return await this.find({
        where: {
          categoryName: categoryName,
          isDeleted: false
        }
      });
    } catch (e) {
      return e;
    }
  }
}