import { EntityRepository, getConnection, Repository } from "typeorm";
import { CreateReeferFriendDto } from "./dto/create-reefer-friend.dto";
import { ReeferFriendEntity } from "./entities/reefer-friend.entity";
import { UpdateReeferFriendDto } from "./dto/update-reefer-friend.dto";

@EntityRepository(ReeferFriendEntity)
export class ReeferFriendRepository extends Repository<ReeferFriendEntity> {
  async createEntity(dto: CreateReeferFriendDto): Promise<ReeferFriendEntity | Error> {
    try {
      const {
        title,
        description
      } = dto;
      const entity = this.create({
        title,
        description
      });
      await this.save(entity);
      return entity;
    } catch (e) {
      return e;
    }
  }

  async findOneEntity(id: string): Promise<ReeferFriendEntity | Error> {
    try {
      return await this.findOne(id);
    } catch (e) {
      return e;
    }
  }

  async findByFilterQuery(query: any): Promise<ReeferFriendEntity[] | Error> {
    try {
      return await this.find({
        where: {
          ...query,
          isDeleted: false
        },
        order: {
          createdAt: "DESC"
        }
      });
    } catch (e) {
      return e;
    }
  }

  async deleteEntity(id: string): Promise<boolean | Error> {
    try {
      const updateResult = await getConnection()
        .createQueryBuilder()
        .update(ReeferFriendEntity)
        .set({ isDeleted: true })
        .where("id = :id", { id: id })
        .execute();
      if (updateResult.affected > 0) {
        return true;
      }
      return false;
    } catch (e) {
      return e;
    }
  }

  async countAllQuery(): Promise<any | Error> {
    try {
      return await this.count({
        where: {
          isDeleted: false
        }
      });
    } catch (e) {
      return e;
    }
  }

  async updateEntity(id: string, dto: UpdateReeferFriendDto): Promise<ReeferFriendEntity | Error> {
    try {
      await this.update(id, {
        title: dto.title,
        description: dto.description
      });
      return await this.findOne(id);
    } catch (e) {
      return e;
    }
  }
}